@@ -44,13 +44,11 @@ LOGGER = logging.getLogger(tests.LOGGER_NAME)
 DEVNULL = open(os.devnull, 'w')
 
 
-# MySQL executables used
+# MySQL Server executable used
 if os.name == 'nt':
     EXEC_MYSQLD = 'mysqld.exe'
-    EXEC_MYSQL = 'mysql.exe'
 else:
     EXEC_MYSQLD = 'mysqld'
-    EXEC_MYSQL = 'mysql'
 
 
 def _convert_forward_slash(path):
@@ -81,6 +79,7 @@ def process_running(pid):
         output = subprocess.check_output("tasklist")
         lines = [line.split(None, 2) for line in output.splitlines() if line]
         for name, apid, _ in lines:
+            name = name.decode('utf-8')
             if name == EXEC_MYSQLD and pid == int(apid):
                 return True
         return False
@@ -117,7 +116,7 @@ def get_pid(pid_file):
     try:
         return int(open(pid_file, 'r').readline().strip())
     except IOError as err:
-        LOGGER.debug("Failed reading pid file: {0}".format(str(err)))
+        LOGGER.debug("Failed reading pid file: %s", err)
         return None
 
 
@@ -137,11 +136,10 @@ class MySQLServerBase(object):
 
     """Base for classes managing a MySQL server"""
 
-    def __init__(self, basedir, option_file=None):
+    def __init__(self, basedir, option_file=None, sharedir=None):
         self._basedir = basedir
-        self._bindir = None
         self._sbindir = None
-        self._sharedir = None
+        self._sharedir = sharedir
         self._scriptdir = None
         self._process = None
         self._lc_messages_dir = None
@@ -162,31 +160,51 @@ class MySQLServerBase(object):
         Raises MySQLBootstrapError when something fails.
         """
 
-        # Locate mysqld, mysql and english/errmsg.sys
+        # Locate mysqld, mysql binaries
+        LOGGER.info("Locating mysql binaries (could take a while)")
         for root, dirs, files in os.walk(self._basedir):
+            if self._sbindir:
+                break
             for afile in files:
                 if (afile == EXEC_MYSQLD and
                         os.access(os.path.join(root, afile), 0)):
                     self._sbindir = root
-                elif (afile == EXEC_MYSQL and
-                        os.access(os.path.join(root, afile), 0)):
-                    self._bindir = root
-                elif (afile == 'errmsg.sys' and 'english' in root):
-                    self._lc_messages_dir = root
-                elif (afile == 'mysql_system_tables.sql'):
+                    break
+
+        if not self._sbindir:
+            raise MySQLBootstrapError(
+                "MySQL binaries not found under {0}. Use --with-mysql".format(
+                    self._basedir))
+
+        # Try to locate errmsg.sys and mysql_system_tables.sql
+        if not self._sharedir:
+            match = self._get_mysqld_help_info(r'^lc-messages-dir\s+(.*)\s*$')
+            if match:
+                self._sharedir = match[0]
+            if not self._sharedir:
+                raise MySQLBootstrapError("Failed getting share folder. "
+                                          "Use --with-mysql-share.")
+        LOGGER.debug("Using share folder: %s", self._sharedir)
+
+        found = False
+        for root, dirs, files in os.walk(self._sharedir):
+            if found:
+                break
+            for afile in files:
+                if afile == 'errmsg.sys' and 'english' in root:
+                    self._lc_messages_dir = self._sharedir
+                elif afile == 'mysql_system_tables.sql':
                     self._scriptdir = root
 
-        if not self._bindir or not self._sbindir:
+        if not self._lc_messages_dir or not self._scriptdir:
             raise MySQLBootstrapError(
-                "MySQL binaries not found under {0}".format(self._basedir))
+                "errmsg.sys and mysql_system_tables.sql not found"
+                " under {0}".format(self._sharedir))
 
-        LOGGER.debug("Location of {bin}: {loc}".format(bin=EXEC_MYSQL,
-                                                       loc=self._bindir))
-        LOGGER.debug("Location of {bin}: {loc}".format(bin=EXEC_MYSQLD,
-                                                       loc=self._sbindir))
+        LOGGER.debug("Location of MySQL Server binaries: %s", self._sbindir)
 
-        LOGGER.debug("Error messages: {loc}".format(loc=self._lc_messages_dir))
-        LOGGER.debug("SQL Script folder: {loc}".format(loc=self._scriptdir))
+        LOGGER.debug("Error messages: %s", self._lc_messages_dir)
+        LOGGER.debug("SQL Script folder: %s", self._scriptdir)
 
     def _get_cmd(self):
         """Returns command to start MySQL server
@@ -203,6 +221,29 @@ class MySQLServerBase(object):
 
         return cmd
 
+    def _get_mysqld_help_info(self, needle):
+        """Get information from the mysqld binary help
+
+        This is basically a grep. Needle is a regular expression which
+        will be looked for in each line of the mysqld --help --verbose
+        output. We return the first match as a list.
+        """
+        cmd = [
+            os.path.join(self._sbindir, EXEC_MYSQLD),
+            '--help', '--verbose'
+        ]
+
+        prc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=DEVNULL)
+        help_verbose = prc.communicate()[0]
+        regex = re.compile(needle)
+        for help_line in help_verbose.splitlines():
+            help_line = help_line.decode('utf-8').strip()
+            match = regex.search(help_line)
+            if match:
+                return match.groups()
+
+        return []
+
     def _get_version(self):
         """Get the MySQL server version
 
@@ -217,7 +258,7 @@ class MySQLServerBase(object):
             '--version'
         ]
 
-        prc = subprocess.Popen(cmd, stdout=subprocess.PIPE)
+        prc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=DEVNULL)
         verstr = str(prc.communicate()[0])
         matches = re.match(r'.*Ver (\d)\.(\d).(\d{1,2}).*', verstr)
         if matches:
@@ -261,7 +302,7 @@ class MySQLServer(MySQLServerBase):
 
     def __init__(self, basedir, topdir, cnf, bind_address, port,
                  name, datadir=None, tmpdir=None,
-                 unix_socket_folder=None, ssl_folder=None):
+                 unix_socket_folder=None, ssl_folder=None, sharedir=None):
         self._cnf = cnf
         self._option_file = os.path.join(topdir, 'my.cnf')
         self._bind_address = bind_address
@@ -280,10 +321,13 @@ class MySQLServer(MySQLServerBase):
         self._install = None
         self._server = None
         self._debug = False
+        self._sharedir = sharedir
 
         self.client_config = {}
 
-        super(MySQLServer, self).__init__(self._basedir, self._option_file)
+        super(MySQLServer, self).__init__(self._basedir,
+                                          self._option_file,
+                                          sharedir=self._sharedir)
 
     def _create_directories(self):
         """Create directory structure for bootstrapping
@@ -294,12 +338,15 @@ class MySQLServer(MySQLServerBase):
 
         Raises MySQLBootstrapError when something fails.
         """
-        LOGGER.debug("Creating {dir} {dir}/mysql and {dir}/test".format(
-            dir=self._datadir))
-        os.mkdir(self._topdir)
-        os.mkdir(os.path.join(self._topdir, 'tmp'))
-        os.mkdir(self._datadir)
-        os.mkdir(os.path.join(self._datadir, 'mysql'))
+        dirs = [
+            self._topdir,
+            os.path.join(self._topdir, 'tmp'),
+            self._datadir,
+            os.path.join(self._datadir, 'mysql')
+        ]
+        for adir in dirs:
+            LOGGER.debug("Creating directory %s", adir)
+            os.mkdir(adir)
 
     def _get_bootstrap_cmd(self):
         """Get the command for bootstrapping.
@@ -324,12 +371,12 @@ class MySQLServer(MySQLServerBase):
             '--tmpdir=%s' % self._tmpdir,
         ]
         if self._version[0:2] < (5, 5):
-            cmd.append('--language={0}'.format(self._lc_messages_dir))
+            cmd.append('--language={0}/english'.format(self._lc_messages_dir))
         else:
             cmd.extend([
                 '--lc-messages-dir={0}'.format(self._lc_messages_dir),
                 '--lc-messages=en_US'
-                ])
+            ])
         return cmd
 
     def bootstrap(self):
@@ -360,15 +407,13 @@ class MySQLServer(MySQLServerBase):
         try:
             self._create_directories()
             cmd = self._get_bootstrap_cmd()
-            sql = []
-            sql.append("USE mysql;")
+            sql = ["USE mysql;"]
             for filename in script_files:
                 full_path = os.path.join(self._scriptdir, filename)
-                LOGGER.debug(
-                    "Reading SQL from '{path}'".format(path=full_path))
+                LOGGER.debug("Reading SQL from '%s'", full_path)
                 with open(full_path, 'r') as fp:
-                    sql += [line.strip() for line in fp.readlines()]
-            sql += extra_sql
+                    sql.extend([line.strip() for line in fp.readlines()])
+            sql.extend(extra_sql)
             logfile = open(os.path.join(self._topdir, 'bootstrap.log'), 'w')
             prc = subprocess.Popen(cmd, stdin=subprocess.PIPE,
                                    stderr=subprocess.STDOUT, stdout=logfile)
@@ -488,8 +533,7 @@ class MySQLServer(MySQLServerBase):
         try:
             rmtree(self._topdir)
         except OSError as err:
-            LOGGER.debug("Failed removing {folder}: {error}".format(
-                folder=self._topdir, error=err))
+            LOGGER.debug("Failed removing %s: %s", self._topdir, err)
             if self._debug is True:
                 raise
         else:
@@ -505,7 +549,7 @@ class MySQLServer(MySQLServerBase):
         """
         pid = pid or get_pid(self._pid_file)
         if pid:
-            LOGGER.debug("Got PID " + str(pid))
+            LOGGER.debug("Got PID %d", pid)
             return process_running(pid)
 
         return False
